use std::fs;
use std::num::NonZeroU32;

use crate::gsa::Gsa;
use crate::gsa_render_to_screen::render_to_screen;
use crate::gsa_render_to_screen::render_to_window;
use crate::sprite::Sprite;
use crate::tileset::*;
use crate::*;
use clap::crate_version;
use glam::IVec2;
use winit::{
    dpi::LogicalSize,
    event::{Event, KeyboardInput, VirtualKeyCode, WindowEvent},
    event_loop::{ControlFlow, EventLoop},
    window::WindowBuilder,
};

pub(crate) fn run_mapedit() {
    println!("running map edit");

    let tileset_path = "examples/basic/gfx.gif";
    let (tileset, palette) = load_tileset(&fs::read(tileset_path).unwrap());

    let mut gsa = Gsa {
        sprites: [Sprite::default(); MAX_SPRITES],
        palette,
        bgs: Default::default(),
        font: FONT_BOLD,
        pressed: 0,
        released: 0,
        down: 0,
    };

    gsa.reset_bgs();
    gsa.reset_sprites();

    let event_loop = EventLoop::new();
    let size = LogicalSize::new(1280, 720);
    let window = WindowBuilder::new()
        .with_title(format!(
            "Game Skunk Advance Map Editor v{}",
            crate_version!()
        ))
        .with_inner_size(size)
        .build(&event_loop)
        .unwrap();

    let context = unsafe { softbuffer::Context::new(&window) }.unwrap();
    let mut surface = unsafe { softbuffer::Surface::new(&context, &window) }.unwrap();
    window.request_redraw();
    let mut mouse_pos = IVec2::ZERO;
    let mut left_down = false;
    event_loop.run(move |event, _, control_flow| {
        let mouse_pos = &mut mouse_pos;
        let left_down = &mut left_down;
        match event {
            Event::WindowEvent { event, .. } => match event {
                WindowEvent::KeyboardInput {
                    input:
                        KeyboardInput {
                            virtual_keycode: Some(VirtualKeyCode::Escape, ..),
                            ..
                        },
                    ..
                } => {
                    *control_flow = ControlFlow::Exit;
                }
                WindowEvent::MouseInput { state, button, .. } => match (state, button) {
                    (winit::event::ElementState::Pressed, winit::event::MouseButton::Left) => {
                        *left_down = true;
                    }
                    (winit::event::ElementState::Released, winit::event::MouseButton::Left) => {
                        *left_down = false;
                    }
                    _ => {}
                },
                WindowEvent::CursorMoved { position, .. } => {
                    *mouse_pos = IVec2 {
                        x: position.x as i32,
                        y: position.y as i32,
                    };
                }
                _ => {}
            },

            Event::MainEventsCleared => {
                if *left_down {
                    let tile_x = mouse_pos.x as usize / TILE_SIZE;
                    let tile_y = mouse_pos.y as usize / TILE_SIZE;
                    gsa.bgs[0].tiles[tile_x][tile_y] = 0;
                }

                // render
                let size = window.inner_size();
                surface
                    .resize(
                        NonZeroU32::new(size.width).unwrap(),
                        NonZeroU32::new(size.height).unwrap(),
                    )
                    .unwrap();
                let mut screen_buffer =
                    vec![TRANSPARENT; size.width as usize * size.height as usize];
                let mut window_buffer = surface.buffer_mut().unwrap();
                render_to_screen(
                    &mut screen_buffer,
                    &gsa,
                    &tileset,
                    IVec2 {
                        x: size.width as i32,
                        y: size.height as i32,
                    },
                );
                render_to_window(
                    &mut window_buffer,
                    &mut screen_buffer,
                    &palette,
                    IVec2 {
                        x: size.width as i32,
                        y: size.height as i32,
                    },
                    1,
                    0,
                    0,
                );
                window_buffer.present().unwrap();
            }
            _ => {}
        }
    });
}
